/*
 * Copyright (C) 2004-2011 Andrey Kholmanskih. All rights reserved.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, write to the 
 *
 * Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330,
 * Boston, MA  USA  02111-1307
 * 
 * 
 * Andrey Kholmanskih
 * support@jdbreport.com
 */
package and.swing;

import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Point;
import java.awt.RenderingHints;
import java.awt.Toolkit;
import java.awt.Window;
import java.awt.geom.PathIterator;
import java.awt.geom.Point2D;
import java.awt.image.BufferedImage;
import java.awt.image.RenderedImage;
import java.util.ArrayList;
import java.util.List;

import javax.swing.ImageIcon;
import javax.swing.JTree;

/**
 * @version 1.6 01.04.2011
 * @author Andrey Kholmanskih
 * 
 */
public class Utils {

	public Utils() {
		super();
	}

	/**
	 * Center screen
	 * 
	 * @param window
	 */
	public static void screenCenter(Window window) {
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		Dimension frameSize = window.getSize();
		if (frameSize.height > screenSize.height) {
			frameSize.height = screenSize.height;
		}
		if (frameSize.width > screenSize.width) {
			frameSize.width = screenSize.width;
		}
		window.setLocation((screenSize.width - frameSize.width) / 2,
				(screenSize.height - frameSize.height) / 2);
	}

	public static void screenCenter(Window window, Window parent) {
		Dimension screenSize;
		Point location;
		if (parent == null) {
			screenSize = Toolkit.getDefaultToolkit().getScreenSize();
			location = new Point(0, 0);
		} else {
			screenSize = parent.getSize();
			location = parent.getLocation();
		}
		Dimension frameSize = window.getSize();
		window.setLocation(location.x + (screenSize.width - frameSize.width)
				/ 2, location.y + (screenSize.height - frameSize.height) / 2);
	}



	public static void setChildFont(Container parent, Font font) {
		for (int i = 0; i < parent.getComponentCount(); i++) {
			Component c = parent.getComponent(i);
			c.setFont(font);
			if (c instanceof Container) {
				setChildFont((Container) c, font);
			}
		}
	}


	public static void expandTree(JTree tree) {
		int row = 0;
		while (row < tree.getRowCount()) {
			tree.expandRow(row);
			row++;
		}
	}

	public static void collapseTree(JTree tree) {
		collapseTree(tree, !tree.isRootVisible());
	}
	
	public static void collapseTree(JTree tree, boolean collapseRoot) {
		int row = tree.getRowCount() - 1;
		int n = collapseRoot ? 0 : 1;
		while (row >= n) {
			tree.collapseRow(row);
			row--;
		}
	}

	public static String colorToString(Color color) {
		return "" + color.getRed() + "," + color.getGreen() + ","
				+ color.getBlue();
	}

	public static Color stringToColor(String color) {
		if (color == null)
			return null;
		int r = 0;
		int g = 0;
		int b = 0;
		int ind1 = color.indexOf(',');
		if (ind1 > 0) {
			r = Integer.parseInt(color.substring(0, ind1));
			int ind2 = color.indexOf(',', ++ind1);
			g = Integer.parseInt(color.substring(ind1, ind2));
			b = Integer.parseInt(color.substring(ind2 + 1));
		} else {
			int c = Integer.parseInt(color);
			b = (c >> 16) & 0xff;
			g = (c >> 8) & 0xff;
			r = c & 0xff;
		}
		return new Color(r, g, b);
	}

	public static List<List<Point2D.Float>> pathToListPoint(PathIterator pi) {
		return  pathToListPoint(pi, 0.2f);
	}
	
	/**
	 * 
	 * @param pi  
	 * @param dt 0 < dt <= 0.5 
	 * @return
	 */
	public static List<List<Point2D.Float>> pathToListPoint(PathIterator pi, float dt) {
		List<List<Point2D.Float>> lp = new ArrayList<List<Point2D.Float>>();
		float[] coords = new float[6];
		Point2D.Float p0;
		
		List<Point2D.Float> points = new ArrayList<Point2D.Float>();
		while (!pi.isDone()) {
			int result = pi.currentSegment(coords);

			switch (result) {
			case PathIterator.SEG_MOVETO:
				points.add(new Point2D.Float(coords[0], coords[1]));
				break;
			case PathIterator.SEG_LINETO:
				points.add(new Point2D.Float(coords[0], coords[1]));
				break;
			case PathIterator.SEG_QUADTO:
				p0 = points.get(points.size() - 1);
				float[] qpx = { p0.x, coords[0], coords[2] };
				float[] qpy = { p0.y, coords[1], coords[3] };

				for (float t = 0; t <= 1; t += dt) {
					Point2D.Float p = new Point2D.Float();
					p.x = and.util.Utilities.quadBezier(t, qpx);
					p.y = and.util.Utilities.quadBezier(t, qpy);
					points.add(p);
				}
				break;
			case PathIterator.SEG_CUBICTO:

				p0 = points.get(points.size() - 1);
				float[] px = { p0.x, coords[0], coords[2], coords[4] };
				float[] py = { p0.y, coords[1], coords[3], coords[5] };
				
				for (float t = 0; t <= 1; t += dt) {
					Point2D.Float p = new Point2D.Float();
					p.x = and.util.Utilities.cubicBezier(t, px);
					p.y = and.util.Utilities.cubicBezier(t, py);
					points.add(p);
				}
				break;
			case PathIterator.SEG_CLOSE:
				lp.add(points);
				points = new ArrayList<Point2D.Float>();
				break;
			}
			pi.next();
		}
		if (points.size() > 0) {
			lp.add(points);
		}
		return lp;
	}

	/**
	 * 
	 * @param icon
	 * @return RendereImage
	 */
	public static RenderedImage getRenderedImage(ImageIcon icon) {
		RenderedImage image = null;
		if (icon.getImage() instanceof RenderedImage) {
			image = (RenderedImage) icon.getImage();
		} else {
			image = new BufferedImage(icon.getIconWidth(), icon
					.getIconHeight(), BufferedImage.TYPE_3BYTE_BGR);
			Graphics2D g = ((BufferedImage) image).createGraphics();
			icon.paintIcon(null, g, 0, 0);
		}
		return image;
	}

    public static Image scaleImage(Image inputImage,
            int scaledWidth, int scaledHeight) {
        BufferedImage tmpImage = new BufferedImage(scaledWidth,
                            scaledHeight, BufferedImage.TYPE_INT_RGB);
       Graphics g = tmpImage.getGraphics();
                    ((Graphics2D)g).setRenderingHint(RenderingHints.KEY_INTERPOLATION, 
                            RenderingHints.VALUE_INTERPOLATION_BILINEAR);
                    g.drawImage(inputImage, 0, 0, tmpImage.getWidth(), 
                            tmpImage.getHeight(), null);
        return tmpImage;
    }

}
